/*
	Copyright (c) 2010 Christiaan Welvaart
	
	Permission is hereby granted, free of charge, to any person obtaining
	a copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, sublicense, and/or sell copies of the Software, and to
	permit persons to whom the Software is furnished to do so, subject to
	the following conditions:

	The above copyright notice and this permission notice shall be included
	in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
	CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
	TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package net.time4t.yarc.ui;

import java.awt.Color;
import java.text.NumberFormat;
import java.util.Vector;
import javax.swing.border.LineBorder;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

class TunerPanel extends InputPanel implements java.awt.event.ActionListener
{
	static final long serialVersionUID = -5147567009754409681L;
	
	private final String[]	search_mode_names = { "Preset", "Tuning" };
	private final String[]	band_names = { "FM", "AM" };
	
	private Device		device;
	private boolean		RDS;
	private boolean		updating;
	
	private JComboBox	search_mode;
	private JComboBox	band;
	private	JTextField	frequency;
	private JButton		freq_up;
	private JButton		freq_down;
	private JLabel		frequency_unit;
	private JComboBox	preset;
	private JButton		preset_up;
	private JButton		preset_down;
	private JTextField	RDS_genre;
	private JTextField	RDS_service;

		TunerPanel(Device device)
	{
		ImageIcon	icon;
		JLabel		label;
		Vector<String>	presetNames;
		int		i;
		BoxLayout	panelLayout;
		JPanel		configPanel;
		BoxLayout	configLayout;
		JPanel		searchModePanel;
		BoxLayout	searchModeLayout;
		JPanel		tuningPanel;
		BoxLayout	tuningLayout;
		JPanel		bandPanel;
		BoxLayout	bandLayout;
		JPanel		bandGluePanel;
		BoxLayout	bandGlueLayout;
		JPanel		frequencyPanel;
		BoxLayout	frequencyLayout;
		JPanel		frequencyUpDownPanel;
		BoxLayout	frequencyUpDownLayout;
		JPanel		presetPanel;
		BoxLayout	presetLayout;
		JPanel		presetUpDownPanel;
		BoxLayout	presetUpDownLayout;
		JPanel		rdsPanel;
		BoxLayout	rdsLayout;
		JPanel		genrePanel;
		BoxLayout	genreLayout;
		JPanel		servicePanel;
		BoxLayout	serviceLayout;
		
		this.device = device;
		updating = false;
		
		detect_rds();
		
		panelLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(panelLayout);
		label = new JLabel("Tuner");
		label.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
		this.add(label);
		
		configPanel = new JPanel();
		configLayout = new BoxLayout(configPanel, BoxLayout.Y_AXIS);
		configPanel.setLayout(configLayout);
		configPanel.setBorder(new LineBorder(Color.BLACK));
		this.add(configPanel);
		
		searchModePanel = new JPanel();
		searchModeLayout = new BoxLayout(searchModePanel, BoxLayout.X_AXIS);
		searchModePanel.setLayout(searchModeLayout);
		searchModePanel.add(new JLabel("Search Mode"));
		search_mode = new JComboBox(search_mode_names);
		searchModePanel.add(search_mode);
		configPanel.add(searchModePanel);
		
		tuningPanel = new JPanel();
		tuningLayout = new BoxLayout(tuningPanel, BoxLayout.X_AXIS);
		tuningPanel.setLayout(tuningLayout);
		
		bandPanel = new JPanel();
		bandLayout = new BoxLayout(bandPanel, BoxLayout.X_AXIS);
		bandPanel.setLayout(bandLayout);
		bandPanel.add(new JLabel("Band"));
		
		bandGluePanel = new JPanel();
		bandGlueLayout = new BoxLayout(bandGluePanel, BoxLayout.Y_AXIS);
		bandGluePanel.setLayout(bandGlueLayout);
		bandGluePanel.add(javax.swing.Box.createVerticalGlue());
		band = new JComboBox(band_names);
		bandGluePanel.add(band);
		bandGluePanel.add(javax.swing.Box.createVerticalGlue());
		bandPanel.add(bandGluePanel);
		tuningPanel.add(bandPanel);
		tuningPanel.add(javax.swing.Box.createHorizontalStrut(5));
		
		frequencyPanel = new JPanel();
		frequencyLayout = new BoxLayout(frequencyPanel, BoxLayout.X_AXIS);
		frequencyPanel.setLayout(frequencyLayout);
		frequencyPanel.add(new JLabel("Frequency"));
		
		frequencyUpDownPanel = new JPanel();
		frequencyUpDownLayout = new BoxLayout(frequencyUpDownPanel, BoxLayout.Y_AXIS);
		frequencyUpDownPanel.setLayout(frequencyUpDownLayout);
		java.net.URL	imageUrl = null;
		imageUrl = TunerPanel.class.getResource("/images/up.gif");
		icon = new ImageIcon(imageUrl);
		freq_up = new JButton(icon);
		freq_up.setMargin(new java.awt.Insets(-4,6,-4,6));
		frequency = new JTextField();
		frequency.setEditable(false);
		imageUrl = TunerPanel.class.getResource("/images/down.gif");
		icon = new ImageIcon(imageUrl);
		freq_down = new JButton(icon);
		freq_down.setMargin(new java.awt.Insets(-4,6,-4,6));
		frequencyUpDownPanel.add(freq_up);
		frequencyUpDownPanel.add(frequency);
		frequencyUpDownPanel.add(freq_down);
		
		frequencyPanel.add(frequencyUpDownPanel);
		frequency_unit = new JLabel("MHz");
		frequencyPanel.add(frequency_unit);
		tuningPanel.add(frequencyPanel);
		tuningPanel.add(javax.swing.Box.createHorizontalStrut(5));
		
		presetPanel = new JPanel();
		presetLayout = new BoxLayout(presetPanel, BoxLayout.X_AXIS);
		presetPanel.setLayout(presetLayout);
		presetPanel.add(new JLabel("Preset"));
		
		presetUpDownPanel = new JPanel();
		presetUpDownLayout = new BoxLayout(presetUpDownPanel, BoxLayout.Y_AXIS);
		presetUpDownPanel.setLayout(presetUpDownLayout);
		imageUrl = TunerPanel.class.getResource("/images/up.gif");
		icon = new ImageIcon(imageUrl);
		preset_up = new JButton(icon);
		preset_up.setMargin(new java.awt.Insets(-4,6,-4,6));
		presetNames = new Vector<String>();
		for (i = 1; i < 41; ++i)
		{
			presetNames.addElement(Integer.toString(i));
		}
		preset = new JComboBox(presetNames);
		imageUrl = TunerPanel.class.getResource("/images/down.gif");
		icon = new ImageIcon(imageUrl);
		preset_down = new JButton(icon);
		preset_down.setMargin(new java.awt.Insets(-4,6,-4,6));
		presetUpDownPanel.add(preset_up);
		presetUpDownPanel.add(preset);
		presetUpDownPanel.add(preset_down);
		presetPanel.add(presetUpDownPanel);
		tuningPanel.add(presetPanel);
		
		configPanel.add(tuningPanel);
		
		if (RDS)
		{
			rdsPanel = new JPanel();
			rdsLayout = new BoxLayout(rdsPanel, BoxLayout.X_AXIS);
			rdsPanel.setLayout(rdsLayout);

			genrePanel = new JPanel();
			genreLayout = new BoxLayout(genrePanel, BoxLayout.X_AXIS);
			genrePanel.setLayout(genreLayout);
			genrePanel.add(new JLabel("Genre:"));
			RDS_genre = new JTextField();
			RDS_genre.setEditable(false);
			genrePanel.add(RDS_genre);
			rdsPanel.add(genrePanel);

			servicePanel = new JPanel();
			serviceLayout = new BoxLayout(servicePanel, BoxLayout.X_AXIS);
			servicePanel.setLayout(serviceLayout);
			servicePanel.add(new JLabel("Service:"));

			RDS_service = new JTextField();
			RDS_service.setEditable(false);
			servicePanel.add(RDS_service);
			rdsPanel.add(servicePanel);
		
			configPanel.add(rdsPanel);

			RDS_genre.addActionListener(this);
			RDS_service.addActionListener(this);
		}
		else
		{
			RDS_genre = null;
			RDS_service = null;
		}
		
		search_mode.addActionListener(this);
		band.addActionListener(this);
		frequency.addActionListener(this);
		freq_up.addActionListener(this);
		freq_down.addActionListener(this);
		preset.addActionListener(this);
		preset_up.addActionListener(this);
		preset_down.addActionListener(this);

		updateStatus(false);
	}
	
	private void detect_rds()
	{
		Document	xml;
		NodeList	nodes;

		xml = device.get("Tuner.Config");
		nodes = xml.getElementsByTagName("RDS");
		RDS = nodes.item(0).getTextContent().equals("Ready");
	}
	
	void updateStatus(boolean enabled)
	{
		Document	xml;
		NodeList	nodes;
		float		val;
		NumberFormat	format;
		int		selectedIndex;
		int		newIndex;
		
		updating = true;
		
		search_mode.setEnabled(enabled);
		band.setEnabled(enabled);
		frequency.setEnabled(enabled);
		preset.setEnabled(enabled);
		if (RDS)
		{
			RDS_genre.setEnabled(enabled);
			RDS_service.setEnabled(enabled);
		}
		
		xml = device.get("Tuner.Play_Info");
		if (xml == null)
		{
			return;
		}
		nodes = xml.getElementsByTagName("Search_Mode");
		if (nodes.getLength() > 0)
		{
			selectedIndex = search_mode.getSelectedIndex();
			newIndex = java.util.Arrays.asList(search_mode_names).indexOf(nodes.item(0).getTextContent());
			if (selectedIndex != newIndex)
			{
				search_mode.setSelectedIndex(newIndex);
			}
		}
		nodes = xml.getElementsByTagName("Preset_Sel");
		if (nodes.getLength() > 0)
		{
			selectedIndex = preset.getSelectedIndex();
			newIndex = Integer.parseInt(nodes.item(0).getTextContent()) - 1;
			if (selectedIndex != newIndex)
			{
				preset.setSelectedIndex(newIndex);
			}
		}
		nodes = xml.getElementsByTagName("Band");
		if (nodes.getLength() > 0)
		{
			selectedIndex = band.getSelectedIndex();
			newIndex = java.util.Arrays.asList(band_names).indexOf(nodes.item(0).getTextContent());
			if (selectedIndex != newIndex)
			{
				band.setSelectedIndex(newIndex);
			}
		}
		
		nodes = xml.getElementsByTagName("Val");
		if (nodes.getLength() > 0)
		{
			val = Integer.parseInt(nodes.item(0).getTextContent());
			nodes = xml.getElementsByTagName("Exp");
			if (nodes.getLength() > 0)
			{
				val /= Math.pow(10.0, Integer.parseInt(nodes.item(0).getTextContent()));
				format = NumberFormat.getInstance();
				format.setMinimumFractionDigits(band.getSelectedIndex() == 0 ? 2 : 0);
				frequency.setText(format.format(val));
			}
		}
		if (RDS)
		{
			nodes = xml.getElementsByTagName("Program_Type");
			if (nodes.getLength() > 0)
			{
				RDS_genre.setText(nodes.item(0).getTextContent());
			}
			nodes = xml.getElementsByTagName("Program_Service");
			if (nodes.getLength() > 0)
			{
				RDS_service.setText(nodes.item(0).getTextContent());
			}
		}
		
		frequency_unit.setText((band.getSelectedIndex() == 0) ? "MHz" : "kHz");

		freq_up.setEnabled(enabled && (search_mode.getSelectedIndex() == 1));
		freq_down.setEnabled(enabled && (search_mode.getSelectedIndex() == 1));
		preset_up.setEnabled(enabled && (search_mode.getSelectedIndex() == 0));
		preset_down.setEnabled(enabled && (search_mode.getSelectedIndex() == 0));
		
		updating = false;
	}

	public void actionPerformed(java.awt.event.ActionEvent e)
	{
		if (!updating)
		{
			if (e.getSource() == search_mode)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Search_Mode", search_mode.getSelectedItem().toString());
				device.put("Tuner.Play_Control", properties);
			}
			else if (e.getSource() == band)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Band", band.getSelectedItem().toString());
				device.put("Tuner.Play_Control.Tuning", properties);
			}
			else if ((e.getSource() == freq_up) || (e.getSource() == freq_down))
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Preset_or_Freq", (e.getSource() == freq_up) ? "Up" : "Down");
				device.put("Tuner.Play_Control", properties);
			}
			else if ((e.getSource() == preset_up) || (e.getSource() == preset_down))
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Preset_or_Freq", (e.getSource() == preset_up) ? "Up" : "Down");
				device.put("Tuner.Play_Control", properties);
			}
			else if (e.getSource() == preset)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Preset_Sel", preset.getSelectedItem().toString());
				device.put("Tuner.Play_Control.Preset", properties);
			}
		}
	}
}

